import os
import json
import boto3
import pytest
from tenacity import retry, stop_after_attempt, wait_fixed

kinesis_client = boto3.client('kinesis', region_name="us-east-1", endpoint_url="https://mockaws:4566",
                              aws_access_key_id="", aws_secret_access_key="", verify=False)

inbound_stream = "psc-uds-customer-ready-alerts-local"
out_local_stream = "local_analytics_output"
out_unittest_stream = "unittest_analytics_output"
data_file_path = "alert-proxy/tests/pytests/alert-proxy/integration_test/data"


@pytest.mark.integration_only
def test_consume_security_event_output():
    sample_protobuf = open(data_file_path + "/protobuf_security_event", "rb")
    kinesis_client.put_record(
        StreamName=inbound_stream,
        Data=sample_protobuf.read(),
        PartitionKey="2"
    )
    stream_record_array = read_from_stream(out_local_stream)
    assert len(stream_record_array) > 0
    json_obj = json_to_obj(stream_record_array)
    assert json_obj["securityEvent"]
    assert json_obj["securityEvent"]["organizationId"] == 1


def json_to_obj(stream_record_array):
    json_obj = json.loads(str(stream_record_array[0]['Data'], 'utf-8'))
    return json_obj


@pytest.mark.integration_only
def test_consume_hbfw_output():
    sample_protobuf = open(data_file_path + "/hbfw_uds_proto", "rb")
    kinesis_client.put_record(
        StreamName=inbound_stream,
        Data=sample_protobuf.read(),
        PartitionKey="2"
    )
    stream_record_array = read_from_stream(out_unittest_stream)
    assert len(stream_record_array) > 0
    json_obj = json.loads(str(stream_record_array[0]['Data'], 'utf-8'))
    assert json_obj["hostBasedFirewallAlarm"]
    assert json_obj["hostBasedFirewallAlarm"]["organizationId"] == 1680137342928


@retry(stop=stop_after_attempt(10), wait=wait_fixed(10))
def read_from_stream(stream_name):
    print("stream name", stream_name)
    describe_stream_response = kinesis_client.describe_stream(StreamName=stream_name)
    shard_response = []
    shards = describe_stream_response['StreamDescription']['Shards']
    for shard in shards:
        shard_id = shard['ShardId']
        response = kinesis_client.get_shard_iterator(
            StreamName=stream_name,
            ShardId=shard_id,
            ShardIteratorType='TRIM_HORIZON'
        )
        shard_iterator = response['ShardIterator']

        max_read = 10
        current_read = 0
        while current_read < max_read and shard_iterator:
            read_response = kinesis_client.get_records(
                ShardIterator=shard_iterator,
                Limit=10
            )
            shard_response = read_response['Records']
            if shard_response:
                for data in shard_response:
                    print("data received ==> ", data)
                break
            else:
                shard_iterator = read_response['NextShardIterator']
                current_read += 1
    assert shard_response
    return shard_response
